<?php

/**
 * AdType captcha class.
 * 
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeCaptcha {
   /**
    * User parameters.
    */
   protected $publicKey;
   
   /**
    * Connection parameters.
    */
   protected $assetsServer;
 
   /**
    * Constructor.
    *
    * @param String $publicKey
    * @param String $assetsServer
    * @access public
    */
   public function __construct($publicKey, $assetsServer)
   {
     $this->publicKey = $publicKey;
     $this->assetsServer = $assetsServer;
   }
   
   /**
    * Returns JavaScript tag for captcha.
    *
    * @param void
    * @return String
    * @access public
    */
   public function render()
   {
     return '<script type="text/javascript" data-key="' . $this->publicKey . '" src="http://'. $this->assetsServer . '/at/widget/challenge.js?k=' . $this->publicKey . '&ts=' . time() . '"></script>';
   }
}

/**
 * AdType user answer class.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeAnswer {
  /**
   * Connection parameters.
   */
  protected $privateKey;
  protected $server;
  
  /**
   * Answer parameters.
   */
  protected $remoteIp;
  protected $challenge;
  protected $answer;
  protected $response;
  
  /**
   * Constructor.
   *
   * @param String $privateKey
   * @param String $server
   * @param String $remoteIp
   * @param String $challenge
   * @param String $answer
   */
  public function __construct($privateKey, $server, $remoteIp, $challenge, $answer)
  {
    $this->privateKey = $privateKey;
    $this->server = $server;
    $this->remoteIp = $remoteIp;
    $this->challenge = $challenge;
    $this->answer = $answer;
  }
  
  /**
   * Checks if captcha is valid.
   *
   * @param void
   * @return Boolean
   * @access public
   */
  public function isValid()
  {
    $request = new AdTypeRequest($this->server, '/widget/verify.php');
    
    $this->response = $request->send(array(
      'adtype_challenge' => $this->challenge,
      'adtype_response' => $this->answer,
      'adtype_private_key' => $this->privateKey,
      'adtype_remote_ip' => $this->remoteIp,
    ));
    
    return $this->response->isValid();
  }
  
  /**
   * Returns response object for answer check.
   *
   * @param void
   * @return AdTypeResponse
   * @access public
   */
  public function getResponse()
  {
    if (empty($this->response)) {
      throw new AdTypeFlowException();
    }
    
    return $this->response;
  }
}

/**
 * AdType backend communication request class.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeRequest {
  /**
   * Request parameters.
   */
  protected $host;
  protected $path;
  protected $port;
  
  /**
   * Constructor.
   *
   * @param String $host
   * @param String $path
   * @param Int $port
   * @access public
   */
  public function __construct($host, $path, $port = 80)
  {
    $this->host = $host;
    $this->path = $path;
    $this->port = $port;
  }
  
  /**
   * Sends data to AdType servers.
   * 
   * @param array of mixed
   * @return AdTypeResponse
   * @access public
   */
  public function send($data)
  {
    $data = http_build_query($data, '', '&');
    
    $request  = "POST $this->path HTTP/1.0\r\n";
    $request .= "Host: $this->host\r\n";
    $request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
    $request .= "Content-Length: " . strlen($data) . "\r\n";
    $request .= "User-Agent: AdType/PHP\r\n";
    $request .= "\r\n";
    $request .= $data;
    
    $response = '';
    
    if (false == ($socket = @fsockopen($this->host, $this->port, $errno, $errstr, 10))) {
      throw new AdTypeCommunicationException();
    }
    
    fwrite($socket, $request);
    
    while (!feof($socket)) {
      $response .= fgets($socket, 1160);
    }
    
    fclose($socket);
    
    $response = explode("\r\n\r\n", $response, 2);
    
    return new AdTypeResponse($response[1]);
  }
}

/**
 * AdType backend response class.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeResponse {
  /**
   * Response parameters.
   */
  protected $isValid;
  protected $message;
  
  /**
   * Constructor.
   *
   * @param AdTypeResponse $response
   * @access public
   */
  public function __construct($response)
  {
    $response = explode("\n", $response);
    
    $this->isValid = ($response[0] == 'OK');
    $this->message = $response[1];
  }
  
  /**
   * Checks if response is valid.
   *
   * @param void
   * @return Boolean
   * @access public
   */
  public function isValid()
  {
    return $this->isValid;
  }
  
  /**
   * Returns response message, if exists.
   *
   * @param void
   * @return String
   * @access public
   */
  public function getMessage()
  {
    return $this->message;
  }
}

/**
 * Base class for exception classes.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeException extends Exception {
}

/**
 * Communication execption class.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeCommunicationException extends AdTypeException {

}

/**
 * Flow exception class.
 *
 * @author LeadBullet SA
 * @package integration.php
 * @version 1.0
 * @since 2012.06.20
 */
class AdTypeFlowException extends AdTypeException {

}